#!/usr/bin/env python
# David D Lowe (c) 2008
# licensed under GPLv3
"""Runs validate(), update_downloads() and update_preview()
validate():Checks every pigment in repoDir to make sure it's state is not STATE_EMPTY or STATE_INCOMPLETE or STATE_INFO,
that all pigments have their data in a data/ folder,
and that all skins have their dependencies in the repo
Exitcode is the number of errors found
update_downloads():Writes downloads.xml
update_preview(): writes preview_summary.xml, preview_summary.tar.gz and fills previewDir
build_pigments(): makes .pigment files"""
# Does not check packs

repoDir = "/var/www/ghns-epidermis0.4/"
repoUrl = ""
previewDir = "preview"
ignoreSymlinkWarnings = False

# TODO: prepare repo for upload
# musn't link to localhost anymore, for example

import sys
import getopt
import os
from xml.dom import minidom
import xml
import subprocess
import pdb



def main():
    _prepare_ep_paths()
    
    stopat = 99
    if len(sys.argv) > 1:
        if sys.argv[1] == "--validate":
            stopat = 1
        elif sys.argv[1] == "--update-downloads":
            stopat = 2
        elif sys.argv[1] == "--update-preview":
            stopat = 3
        elif sys.argv[1] == "--build-pigments":
            stopat = 99
        else:
            print >> sys.stderr, "unrecognised argument: " + sys.argv[1]
            print >> sys.stderr, "reconginsed argumnts are: --validate , --update-downloads , --update-preview, --build-pigments"
    
    print "validating pigments..."
    errors, pigments = validate()
        
    if errors > 0:
        print >> sys.stderr, "found %s error(s)" % errors

        sys.exit(errors)
    else:
        print "no errors found, preliminary check passed"

    if stopat > 1:
        update_downloads(pigments)
        
    if stopat > 2:
        update_preview(pigments)
        
    if stopat > 3:
        build_pigments(pigments)

def validate():
    """returns (errors, pigments)"""
    import const
    import managepigments
    import pigment
    
    errors = 0
    
    readPigments = []
    for pigmentType in const.PIGMENT_TYPES[1:] + ["skin"]:
        plural = managepigments.get_pigment_type(pigmentType).directoryName
        for dir in os.listdir(os.path.join(repoDir, plural)):
            if os.path.isdir(os.path.join(repoDir,plural,dir)):
                curr = os.path.join(repoDir, plural, dir)
                if os.path.exists(os.path.join(curr,"pigment.xml")):
                    pigm = managepigments.create_pigment(pigmentType)
                    try:
                        if pigmentType == "skin":
                            pigm.read_with_pigments(os.path.join(curr, "pigment.xml"), readPigments)
                        else:
                            pigm.read(os.path.join(curr, "pigment.xml"))
                    except Exception, ee:
                        print "error: " + os.path.join(curr, "pigment.xml") + " cannot be read, following exception:"
                        print "::", str(ee)
                        errors += 1
                    else:
                        newErrors = 0
                        if not os.path.abspath(os.path.join(repoDir, plural, pigm.codeName)) == curr:
                            print "error: " + os.path.join(curr, "pigment.xml") + "'s directory does not match its codeName: " + pigm.codeName
                            newErrors += 1
                        elif not os.path.isdir(os.path.join(curr, "data")):
                            print "error: Cannot find directory " + os.path.join(curr, "data")
                            newErrors += 1
                        else:
                            for attach in pigm.attachments:
                                justFilename = attach[(-1*len(attach.split("/")[-1])):]
                                if not os.path.exists(os.path.join(curr, "data", justFilename)):
                                    print "error: " + os.path.join(curr, "pigment.xml") + " cannot be read, cannot find the following attachment in data:"
                                    print "::", "\"" + justFilename + "\""
                                    newErrors += 1
                                elif not os.access(os.path.join(curr, "data", justFilename), os.R_OK):
                                    print "error: " + os.path.join(curr, "pigment.xml") + " cannot be read, permission error for following attachment:"
                                    print "::", "\"" + justFilename + "\""
                                    newErrors += 1
                            for parent, dirs, files in os.walk(curr):
                                if not os.access(os.path.realpath(parent), os.R_OK + os.X_OK):
                                    if not os.path.isdir(os.path.realpath(parent)):
                                        if not ignoreSymlinkWarnings:
                                            print "warning: " + parent + " is a symlink that points to a directory that does not exist"
                                    elif os.stat(parent).st_uid != 0:
                                        print "error: " + parent + " is not owned by root"
                                        newErrors +=1
                                        continue
                                    else:
                                        print "error: " + parent + " cannot be read, permissions problem"
                                        newErrors += 1
                                        continue
                                for file in files:
                                    if not os.access(os.path.realpath(os.path.join(parent, file)), os.R_OK):
                                        if not os.path.exists(os.path.realpath(os.path.join(parent, file))):
                                            if not ignoreSymlinkWarnings:
                                                print "warning: " + os.path.join(parent, file) + " is a symlink that points to a file that does not exist"
                                        elif os.stat(os.path.join(parent, file)).st_uid != 0:
                                            print "error: " + os.path.join(parent, file) + " is not owned by root"
                                            newErrors += 1
                                        else:
                                            print "error: " + os.path.join(parent, file) + " cannot be read, permissions problem"
                                            newErrors += 1

                            
                        if newErrors == 0:
                            if pigm.get_state() in (pigment.STATE_EMPTY, pigment.STATE_INCOMPLETE):
                                print "error: " + os.path.join(curr, "pigment.xml") + " is " + pigm.debug_get_state_str()
                                errors += 1
                            elif not pigm.is_attachment_correct():
                                print "error: " + os.path.join(curr, "pigment.xml") + " has an invalid attachment"
                                errors += 1
                        else:
                            errors += newErrors
                        if pigm.copyright.lower() == "unknown":
                            print "warning: " + os.path.join(curr, "pigment.xml") + " has unknown copyright"
                        readPigments.append(pigm)
                else:
                    print "error: missing " + os.path.join(curr,"pigment.xml")
                    errors += 1
                    
    

    return (errors, readPigments)


def update_downloads(pigments):
    """pigments: list of Pigment objects"""
    import const
    import managepigments
    import pigment
    from xml.dom import minidom
    
    docDownloads = minidom.Document()
    rootDownloads = docDownloads.createElement("ghnsdownload")
    rootDownloads.setAttribute("epidermisVersion","0.4")
    docDownloads.appendChild(rootDownloads)
    
    stuffPreview = docDownloads.createElement("stuff")
    stuffPreview.setAttribute("category", "epidermis/repo_preview_summary")
    nn = docDownloads.createElement("name")
    nn.appendChild(docDownloads.createTextNode("Summary"))
    pp = docDownloads.createElement("payload")
    pp.appendChild(docDownloads.createTextNode(repoUrl + "preview_summary.tar.gz"))
    for ii in (nn, pp):
        stuffPreview.appendChild(ii)
    rootDownloads.appendChild(stuffPreview) 
    
    for pigm in pigments:
        stuff = docDownloads.createElement("stuff")
        stuff.setAttribute("category", "epidermis/pigment/" + pigm.type.codeName)
        for lang in pigm.humanName.keys():
            name = docDownloads.createElement("name")
            name.setAttribute("lang", lang)
            name.appendChild(docDownloads.createTextNode(pigm.get_human_name(lang)))
            stuff.appendChild(name)
        author = docDownloads.createElement("author")
        author.appendChild(docDownloads.createTextNode(pigm.author))
        licence = docDownloads.createElement("licence")
        licence.appendChild(docDownloads.createTextNode(pigm.copyright))
        version = docDownloads.createElement("version")
        version.appendChild(docDownloads.createTextNode("1.0"))
        release = docDownloads.createElement("release")
        release.appendChild(docDownloads.createTextNode("1"))
        releasedate = docDownloads.createElement("releasedate")
        payload = docDownloads.createElement("payload")
        payload.appendChild(docDownloads.createTextNode(\
            os.path.join(repoDir, pigm.type.directoryName, pigm.codeName, pigm.codeName + "_" + pigm.type.codeName + ".pigment")))
        for ii in (author, licence, version, release, releasedate, payload):
            stuff.appendChild(ii)
        rootDownloads.appendChild(stuff)
    
    fileDownloads = open(os.path.join(repoDir, "downloads.xml"), "w")
    docDownloads.writexml(fileDownloads)
    fileDownloads.close()

    print "Wrote %s" % os.path.join(repoDir, "downloads.xml")
    
    
    
def update_preview(pigments):
    """pigments: a list of Pigment objects"""
    import const
    import managepigments
    import pigment
    from xml.dom import minidom
    
    docSummary = minidom.Document()
    rootSummary = docSummary.createElement("previewSummary")
    docSummary.appendChild(rootSummary)
    
    ptElement = {}
    for pt in const.PIGMENT_TYPES:
        ptElement[pt] = docSummary.createElement(managepigments.get_pigment_type(pt).directoryName)
        rootSummary.appendChild(ptElement[pt])
    
    for pigm in pigments:
        item = docSummary.createElement("item")
        item.setAttribute("id", pigm.type.abbreviation + "-" + pigm.codeName)
        for lang in pigm.humanName.keys():
            name = docSummary.createElement("name")
            name.setAttribute("lang", lang)
            name.appendChild(docSummary.createTextNode(pigm.get_human_name(lang)))
            item.appendChild(name)
        pigmentEl = docSummary.createElement("codename")
        pigmentEl.appendChild(docSummary.createTextNode(pigm.codeName))
        copyright = docSummary.createElement("copyright")
        copyright.appendChild(docSummary.createTextNode(pigm.copyright))
        for it in (pigmentEl, copyright):
            item.appendChild(it)
        for lang in pigm.description.keys():
            description = docSummary.createElement("description")
            description.setAttribute("lang", lang)
            description.appendChild(docSummary.createTextNode(pigm.get_description(lang)))
            item.appendChild(description)
        author = docSummary.createElement("author")
        author.appendChild(docSummary.createTextNode(pigm.author))
        item.appendChild(author)
        if pigm.type.codeName == "skin":
            for pt2 in pigm.linkedPigments.keys():
                if pt2 in const.PIGMENT_TYPES[1:]:
                    link = docSummary.createElement(pt2 + "Dependancy")
                    link.appendChild(docSummary.createTextNode(pigm.linkedPigments[pt2].codeName))
                    item.appendChild(link)
        preview = docSummary.createElement("preview")
        preview.appendChild(docSummary.createTextNode(previewDir + "/" + pigm.type.directoryName + \
            "/" + pigm.codeName + "/" + pigm.previewFilename))
        item.appendChild(preview)
        ptElement[pigm.type.codeName].appendChild(item)
    
    fileSummary = open(os.path.join(repoDir, "preview_summary.xml"), "w")
    docSummary.writexml(fileSummary)
    fileSummary.close()

    print "Wrote %s" % os.path.join(repoDir, "preview_summary.xml")    
    
    import subprocess
    subprocess.call(["rm", "-f", "-r", os.path.join(repoDir, previewDir)])
    subprocess.call(["mkdir", "-p", os.path.join(repoDir, previewDir)])
    for pt in const.PIGMENT_TYPES:
        subprocess.call(["mkdir", "-p", os.path.join(repoDir, previewDir, managepigments.get_pigment_type(pt).directoryName)])
    for pigm in pigments:
        if len(pigm.previewFilename) > 0:
            subprocess.call(["mkdir", "-p", os.path.join(repoDir, previewDir, pigm.type.directoryName, pigm.codeName)])
            subprocess.call(["cp", os.path.join(repoDir, pigm.type.directoryName, pigm.codeName, pigm.previewFilename), \
                os.path.join(repoDir, previewDir, pigm.type.directoryName, pigm.codeName)])
    
    subprocess.call(["tar", "-cf", "preview_summary.tar", "preview_summary.xml", previewDir], cwd=repoDir)
    subprocess.call(["rm", "-f", "preview_summary.tar.gz"], cwd=repoDir)
    subprocess.call(["gzip", "preview_summary.tar"], cwd=repoDir)
    
    print "Finished %s/ and preview_summary.tar.gz" % previewDir

def build_pigments(pigments):
    """Builds .pigment files from the pigment.xml, preview and data in the repo
    pigments: list of Pigment objects in the repo"""
    
    import const
    import managepigments
    
    
    for pigm in pigments:
        curr = os.path.join(repoDir, pigm.type.directoryName, pigm.codeName)
        sys.stdout.write(os.path.join(curr, pigm.codeName + "_" + pigm.type.codeName + ".pigment") + " ... ")
        
        try:
            subprocess.call(["mkdir", "-p", "/tmp/epidermis/build_pigment/"])
            subprocess.call("rm -r -f /tmp/epidermis/build_pigment/*", shell=True)
            if len(pigm.previewFilename) > 0:
                subprocess.call(["cp", "-r", "data/", "pigment.xml", pigm.previewFilename, "/tmp/epidermis/build_pigment"], cwd=curr)
            else:
                subprocess.call(["cp", "-r", "data/", "pigment.xml", "/tmp/epidermis/build_pigment"], cwd=curr)
            subprocess.call(["mkdir", "/tmp/epidermis/build_pigment/information"])
            subprocess.call(["mv", "pigment.xml", "information/"], cwd="/tmp/epidermis/build_pigment")
            if len(pigm.previewFilename) > 0:
                subprocess.call(["mv", pigm.previewFilename, "information/"], cwd="/tmp/epidermis/build_pigment")
            subprocess.call(["tar", "-cf", "build.tar", "data/", "information/"], cwd="/tmp/epidermis/build_pigment")
            subprocess.call(["gzip", "build.tar"], cwd="/tmp/epidermis/build_pigment")
            subprocess.call(["mv", "-f", os.path.join("/tmp/epidermis/build_pigment", "build.tar.gz"), \
                os.path.join(curr, pigm.codeName + "_" + pigm.type.codeName + ".pigment")])
        except Exception, ee:
            print ""
            raise(ee)
        else:
            print "built"
    

def _prepare_ep_paths():
    ep_path = determine_path()[:(-1*len(determine_path().split("/")[-1]))]
    sys.path.append(os.path.join(ep_path, "epidermis"))
    sys.path.append(os.path.join(ep_path, "epidermis", "pigments"))


def determine_path ():
    """Borrowed from wxglade.py"""
    try:
        root = __file__
        if os.path.islink (root):
            root = os.path.realpath (root)
        return os.path.dirname (os.path.abspath (root))
    except Exception, ee:
        print >> sys.stderr, "I'm sorry, but something is wrong."
        print >> sys.stderr, "There is no __file__ variable. Please contact the author."
        sys.exit (1)
        raise(ee)

if __name__ == "__main__":
    main()
                            
